import typing_extensions

from openapi_client.paths import PathValues
from openapi_client.apis.paths.j_security_check import JSecurityCheck
from openapi_client.apis.paths.dataservice_client_token import DataserviceClientToken
from openapi_client.apis.paths.dataservice_device import DataserviceDevice
from openapi_client.apis.paths.dataservice_device_monitor import DataserviceDeviceMonitor
from openapi_client.apis.paths.dataservice_device_counters import DataserviceDeviceCounters
from openapi_client.apis.paths.dataservice_statistics_interface import DataserviceStatisticsInterface
from openapi_client.apis.paths.dataservice_template_feature import DataserviceTemplateFeature
from openapi_client.apis.paths.dataservice_template_feature_types import DataserviceTemplateFeatureTypes
from openapi_client.apis.paths.dataservice_template_policy_vedge_devices import DataserviceTemplatePolicyVedgeDevices
from openapi_client.apis.paths.dataservice_template_policy_list import DataserviceTemplatePolicyList

PathToApi = typing_extensions.TypedDict(
    'PathToApi',
    {
        PathValues.J_SECURITY_CHECK: JSecurityCheck,
        PathValues.DATASERVICE_CLIENT_TOKEN: DataserviceClientToken,
        PathValues.DATASERVICE_DEVICE: DataserviceDevice,
        PathValues.DATASERVICE_DEVICE_MONITOR: DataserviceDeviceMonitor,
        PathValues.DATASERVICE_DEVICE_COUNTERS: DataserviceDeviceCounters,
        PathValues.DATASERVICE_STATISTICS_INTERFACE: DataserviceStatisticsInterface,
        PathValues.DATASERVICE_TEMPLATE_FEATURE: DataserviceTemplateFeature,
        PathValues.DATASERVICE_TEMPLATE_FEATURE_TYPES: DataserviceTemplateFeatureTypes,
        PathValues.DATASERVICE_TEMPLATE_POLICY_VEDGE_DEVICES: DataserviceTemplatePolicyVedgeDevices,
        PathValues.DATASERVICE_TEMPLATE_POLICY_LIST: DataserviceTemplatePolicyList,
    }
)

path_to_api = PathToApi(
    {
        PathValues.J_SECURITY_CHECK: JSecurityCheck,
        PathValues.DATASERVICE_CLIENT_TOKEN: DataserviceClientToken,
        PathValues.DATASERVICE_DEVICE: DataserviceDevice,
        PathValues.DATASERVICE_DEVICE_MONITOR: DataserviceDeviceMonitor,
        PathValues.DATASERVICE_DEVICE_COUNTERS: DataserviceDeviceCounters,
        PathValues.DATASERVICE_STATISTICS_INTERFACE: DataserviceStatisticsInterface,
        PathValues.DATASERVICE_TEMPLATE_FEATURE: DataserviceTemplateFeature,
        PathValues.DATASERVICE_TEMPLATE_FEATURE_TYPES: DataserviceTemplateFeatureTypes,
        PathValues.DATASERVICE_TEMPLATE_POLICY_VEDGE_DEVICES: DataserviceTemplatePolicyVedgeDevices,
        PathValues.DATASERVICE_TEMPLATE_POLICY_LIST: DataserviceTemplatePolicyList,
    }
)



# do not import all endpoints into this module because that uses a lot of memory and stack frames
# if you need the ability to import all endpoints from this module, import them with
# from openapi_client.apis.tag_to_api import tag_to_api

import enum


class TagValues(str, enum.Enum):
    AUTHENTICATION = "Authentication"
    SDWAN_FABRIC_DEVICES = "SD-WAN Fabric Devices"
    SDWAN_DEVICE_TEMPLATE = "SD-WAN Device Template"
    SDWAN_DEVICE_POLICY = "SD-WAN Device Policy"



# coding: utf-8

"""


    Generated by: https://openapi-generator.tech
"""

from dataclasses import dataclass
import typing_extensions
import urllib3
from urllib3._collections import HTTPHeaderDict

from openapi_client import api_client, exceptions
from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from openapi_client import schemas  # noqa: F401

from . import path

# header params
ContentTypeSchema = schemas.StrSchema
RequestRequiredHeaderParams = typing_extensions.TypedDict(
    'RequestRequiredHeaderParams',
    {
    }
)
RequestOptionalHeaderParams = typing_extensions.TypedDict(
    'RequestOptionalHeaderParams',
    {
        'Content-Type': typing.Union[ContentTypeSchema, str, ],
    },
    total=False
)


class RequestHeaderParams(RequestRequiredHeaderParams, RequestOptionalHeaderParams):
    pass


request_header_content_type = api_client.HeaderParameter(
    name="Content-Type",
    style=api_client.ParameterStyle.SIMPLE,
    schema=ContentTypeSchema,
)
# body param


class SchemaForRequestBodyApplicationXWwwFormUrlencoded(
    schemas.AnyTypeSchema,
):


    class MetaOapg:
        
        class properties:
            j_username = schemas.StrSchema
            j_password = schemas.StrSchema
            __annotations__ = {
                "j_username": j_username,
                "j_password": j_password,
            }

    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["j_username"]) -> MetaOapg.properties.j_username: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["j_password"]) -> MetaOapg.properties.j_password: ...
    
    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema: ...
    
    def __getitem__(self, name: typing.Union[typing_extensions.Literal["j_username", "j_password", ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)
    
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["j_username"]) -> typing.Union[MetaOapg.properties.j_username, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["j_password"]) -> typing.Union[MetaOapg.properties.j_password, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: str) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]: ...
    
    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal["j_username", "j_password", ], str]):
        return super().get_item_oapg(name)
    

    def __new__(
        cls,
        *args: typing.Union[dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, ],
        j_username: typing.Union[MetaOapg.properties.j_username, str, schemas.Unset] = schemas.unset,
        j_password: typing.Union[MetaOapg.properties.j_password, str, schemas.Unset] = schemas.unset,
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
    ) -> 'SchemaForRequestBodyApplicationXWwwFormUrlencoded':
        return super().__new__(
            cls,
            *args,
            j_username=j_username,
            j_password=j_password,
            _configuration=_configuration,
            **kwargs,
        )


request_body_body = api_client.RequestBody(
    content={
        'application/x-www-form-urlencoded': api_client.MediaType(
            schema=SchemaForRequestBodyApplicationXWwwFormUrlencoded),
    },
)


@dataclass
class ApiResponseFor200(api_client.ApiResponse):
    response: urllib3.HTTPResponse
    body: typing.Union[
        schemas.Unset,
    ]
    headers: schemas.Unset = schemas.unset


_response_for_200 = api_client.OpenApiResponse(
    response_cls=ApiResponseFor200,
    content={
        'application/json': api_client.MediaType(),
    },
)
_status_code_to_response = {
    '200': _response_for_200,
}
_all_accept_content_types = (
    'application/json',
)


class BaseApi(api_client.Api):

    def _j_security_check_post_oapg(
        self: api_client.Api,
        body: typing.Union[SchemaForRequestBodyApplicationXWwwFormUrlencoded, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, schemas.Unset] = schemas.unset,
        header_params: RequestHeaderParams = frozendict.frozendict(),
        content_type: str = 'application/x-www-form-urlencoded',
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
        skip_deserialization: bool = False,
    ) -> typing.Union[
        ApiResponseFor200,
        api_client.ApiResponseWithoutDeserialization
    ]:
        """
        Authentication
        :param skip_deserialization: If true then api_response.response will be set but
            api_response.body and api_response.headers will not be deserialized into schema
            class instances
        """
        self._verify_typed_dict_inputs_oapg(RequestHeaderParams, header_params)
        used_path = path.value

        _headers = HTTPHeaderDict()
        for parameter in (
            request_header_content_type,
        ):
            parameter_data = header_params.get(parameter.name, schemas.unset)
            if parameter_data is schemas.unset:
                continue
            serialized_data = parameter.serialize(parameter_data)
            _headers.extend(serialized_data)
        # TODO add cookie handling
        if accept_content_types:
            for accept_content_type in accept_content_types:
                _headers.add('Accept', accept_content_type)

        _fields = None
        _body = None
        if body is not schemas.unset:
            serialized_data = request_body_body.serialize(body, content_type)
            _headers.add('Content-Type', content_type)
            if 'fields' in serialized_data:
                _fields = serialized_data['fields']
            elif 'body' in serialized_data:
                _body = serialized_data['body']
        response = self.api_client.call_api(
            resource_path=used_path,
            method='post'.upper(),
            headers=_headers,
            fields=_fields,
            body=_body,
            stream=stream,
            timeout=timeout,
        )

        if skip_deserialization:
            api_response = api_client.ApiResponseWithoutDeserialization(response=response)
        else:
            response_for_status = _status_code_to_response.get(str(response.status))
            if response_for_status:
                api_response = response_for_status.deserialize(response, self.api_client.configuration)
            else:
                api_response = api_client.ApiResponseWithoutDeserialization(response=response)

        if not 200 <= response.status <= 299:
            raise exceptions.ApiException(api_response=api_response)

        return api_response


class JSecurityCheckPost(BaseApi):
    # this class is used by api classes that refer to endpoints with operationId fn names

    def j_security_check_post(
        self: BaseApi,
        body: typing.Union[SchemaForRequestBodyApplicationXWwwFormUrlencoded, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, schemas.Unset] = schemas.unset,
        header_params: RequestHeaderParams = frozendict.frozendict(),
        content_type: str = 'application/x-www-form-urlencoded',
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
        skip_deserialization: bool = False,
    ) -> typing.Union[
        ApiResponseFor200,
        api_client.ApiResponseWithoutDeserialization
    ]:
        return self._j_security_check_post_oapg(
            body=body,
            header_params=header_params,
            content_type=content_type,
            accept_content_types=accept_content_types,
            stream=stream,
            timeout=timeout,
            skip_deserialization=skip_deserialization
        )


class ApiForpost(BaseApi):
    # this class is used by api classes that refer to endpoints by path and http method names

    def post(
        self: BaseApi,
        body: typing.Union[SchemaForRequestBodyApplicationXWwwFormUrlencoded, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, schemas.Unset] = schemas.unset,
        header_params: RequestHeaderParams = frozendict.frozendict(),
        content_type: str = 'application/x-www-form-urlencoded',
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
        skip_deserialization: bool = False,
    ) -> typing.Union[
        ApiResponseFor200,
        api_client.ApiResponseWithoutDeserialization
    ]:
        return self._j_security_check_post_oapg(
            body=body,
            header_params=header_params,
            content_type=content_type,
            accept_content_types=accept_content_types,
            stream=stream,
            timeout=timeout,
            skip_deserialization=skip_deserialization
        )





# coding: utf-8

"""
    Cisco-Reservable-SD-WAN

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""

from openapi_client.paths.dataservice_template_policy_list.get import DataserviceTemplatePolicyListGet
from openapi_client.paths.dataservice_template_policy_vedge_devices.get import DataserviceTemplatePolicyVedgeDevicesGet


class SDWANDevicePolicyApi(
    DataserviceTemplatePolicyListGet,
    DataserviceTemplatePolicyVedgeDevicesGet,
):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """
    pass



import typing_extensions

from openapi_client.apis.tags import TagValues
from openapi_client.apis.tags.authentication_api import AuthenticationApi
from openapi_client.apis.tags.sdwan_fabric_devices_api import SDWANFabricDevicesApi
from openapi_client.apis.tags.sdwan_device_template_api import SDWANDeviceTemplateApi
from openapi_client.apis.tags.sdwan_device_policy_api import SDWANDevicePolicyApi

TagToApi = typing_extensions.TypedDict(
    'TagToApi',
    {
        TagValues.AUTHENTICATION: AuthenticationApi,
        TagValues.SDWAN_FABRIC_DEVICES: SDWANFabricDevicesApi,
        TagValues.SDWAN_DEVICE_TEMPLATE: SDWANDeviceTemplateApi,
        TagValues.SDWAN_DEVICE_POLICY: SDWANDevicePolicyApi,
    }
)

tag_to_api = TagToApi(
    {
        TagValues.AUTHENTICATION: AuthenticationApi,
        TagValues.SDWAN_FABRIC_DEVICES: SDWANFabricDevicesApi,
        TagValues.SDWAN_DEVICE_TEMPLATE: SDWANDeviceTemplateApi,
        TagValues.SDWAN_DEVICE_POLICY: SDWANDevicePolicyApi,
    }
)



# coding: utf-8

"""
    Cisco-Reservable-SD-WAN

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""

from openapi_client.paths.dataservice_device_counters.get import DataserviceDeviceCountersGet
from openapi_client.paths.dataservice_device.get import DataserviceDeviceGet
from openapi_client.paths.dataservice_device_monitor.get import DataserviceDeviceMonitorGet
from openapi_client.paths.dataservice_statistics_interface.get import DataserviceStatisticsInterfaceGet


class SDWANFabricDevicesApi(
    DataserviceDeviceCountersGet,
    DataserviceDeviceGet,
    DataserviceDeviceMonitorGet,
    DataserviceStatisticsInterfaceGet,
):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """
    pass



# do not import all endpoints into this module because that uses a lot of memory and stack frames
# if you need the ability to import all endpoints from this module, import them with
# from openapi_client.paths.dataservice_template_feature_types import Api

from openapi_client.paths import PathValues

path = PathValues.DATASERVICE_TEMPLATE_FEATURE_TYPES


# do not import all endpoints into this module because that uses a lot of memory and stack frames
# if you need the ability to import all endpoints from this module, import them with
# from openapi_client.apis.path_to_api import path_to_api

import enum


class PathValues(str, enum.Enum):
    J_SECURITY_CHECK = "/j_security_check"
    DATASERVICE_CLIENT_TOKEN = "/dataservice/client/token"
    DATASERVICE_DEVICE = "/dataservice/device"
    DATASERVICE_DEVICE_MONITOR = "/dataservice/device/monitor"
    DATASERVICE_DEVICE_COUNTERS = "/dataservice/device/counters"
    DATASERVICE_STATISTICS_INTERFACE = "/dataservice/statistics/interface"
    DATASERVICE_TEMPLATE_FEATURE = "/dataservice/template/feature"
    DATASERVICE_TEMPLATE_FEATURE_TYPES = "/dataservice/template/feature/types"
    DATASERVICE_TEMPLATE_POLICY_VEDGE_DEVICES = "/dataservice/template/policy/vedge/devices"
    DATASERVICE_TEMPLATE_POLICY_LIST = "/dataservice/template/policy/list"
